#include "customble.h"
#include "app_timer.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_srv_common.h"
#include "bsp.h"
#include "nordic_common.h"
#include "nrf_ble_gatt.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_sdh.h"
#include "nrf_sdh_ble.h"
#include "nrf_soc.h"
#include <stdbool.h>
#include <stdint.h>
#include <string.h>

#define DEVICE_NAME "T"
#define APP_BLE_CONN_CFG_TAG 1
#define NON_CONNECTABLE_ADV_INTERVAL MSEC_TO_UNITS(100, UNIT_0_625_MS)
#define CONNECTABLE_ADV_INTERVAL MSEC_TO_UNITS(50, UNIT_0_625_MS)
#define APP_ADV_INTERVAL MSEC_TO_UNITS(100, UNIT_0_625_MS) // 100 ms
#define APP_ADV_TIMEOUT_IN_SECONDS 0
#define MIN_DESIRED_MTU_SIZE 23;
#define MIN_CONN_INTERVAL MSEC_TO_UNITS(7.5, UNIT_1_25_MS) // 7.5 ms
#define MAX_CONN_INTERVAL MSEC_TO_UNITS(50, UNIT_1_25_MS)  // 50 ms
#define SLAVE_LATENCY 4                                    /**< Slave latency. */
#define CONN_SUP_TIMEOUT MSEC_TO_UNITS(6000, UNIT_10_MS)   // 6 seconds
#define SLEEP_IN_MS 10000

BLE_ADVERTISING_DEF(m_advertising);

APP_TIMER_DEF(m_awake_timer_id);
APP_TIMER_DEF(m_sleep_timer_id);

void update_advertising_data_while_advertising();

void awake_timer_handler()
{
  update_advertising_data_while_advertising();
  advertising_start();
}

void sleep_timer_handler()
{
  sd_ble_gap_adv_stop(m_advertising.adv_handle);

  // Proceed with setting the device to low power mode
  app_timer_start(m_awake_timer_id, APP_TIMER_TICKS(SLEEP_IN_MS), NULL); // 60 seconds
}

void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
  switch (ble_adv_evt)
  {
  case BLE_ADV_EVT_IDLE:
    sleep_timer_handler();
    break;
  }
}

void advertising_init(void)
{
  uint32_t err_code;
  ble_advdata_t advdata;
  uint8_t flags = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;

  // Dummy data for sensor_data
  uint8_t sensor_data[4] = {0x01, 0x02, 0x03, 0x04};

  // Prepare manufacturer specific data with encrypted data
  ble_advdata_manuf_data_t manuf_specific_data;
  manuf_specific_data.data.p_data = sensor_data;
  manuf_specific_data.data.size = sizeof(sensor_data);

  // Initialize advertising module
  ble_advertising_init_t init;
  memset(&init, 0, sizeof(init));

  init.advdata.name_type = BLE_ADVDATA_NO_NAME;
  init.advdata.include_ble_device_addr = false;
  init.advdata.include_appearance = false;
  init.advdata.flags = flags;
  init.advdata.p_manuf_specific_data = &manuf_specific_data;
  init.config.ble_adv_fast_enabled = true;
  init.config.ble_adv_fast_interval = NON_CONNECTABLE_ADV_INTERVAL;
  init.config.ble_adv_fast_timeout = 180000;
  //init.config.ble_adv_fast_timeout = 100;

  init.evt_handler = on_adv_evt;

  err_code = ble_advertising_init(&m_advertising, &init);
  APP_ERROR_CHECK(err_code);

  // Set the TX power level
  err_code = sd_ble_gap_tx_power_set(BLE_GAP_TX_POWER_ROLE_ADV, m_advertising.adv_handle, RADIO_TXPOWER_TXPOWER_Pos4dBm);
  APP_ERROR_CHECK(err_code);

  ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}

/**@brief Function for starting advertising.
 */
void advertising_start(void)
{
  ret_code_t err_code;

  err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
  APP_ERROR_CHECK(err_code);
}

/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
void ble_stack_init(void)
{
  ret_code_t err_code;

  err_code = nrf_sdh_enable_request();
  APP_ERROR_CHECK(err_code);

  // Configure the BLE stack using the default settings.
  // Fetch the start address of the application RAM.
  uint32_t ram_start = 0;
  err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
  APP_ERROR_CHECK(err_code);

  // Enable BLE stack.
  err_code = nrf_sdh_ble_enable(&ram_start);
  APP_ERROR_CHECK(err_code);
}

void ble_timers_init(void)
{
  ret_code_t err_code;

  // Create timers
  err_code = app_timer_create(&m_awake_timer_id, APP_TIMER_MODE_SINGLE_SHOT, awake_timer_handler);
  APP_ERROR_CHECK(err_code);

  err_code = app_timer_create(&m_sleep_timer_id, APP_TIMER_MODE_SINGLE_SHOT, sleep_timer_handler);
  APP_ERROR_CHECK(err_code);
}

void update_advertising_data_while_advertising()
{
  ret_code_t err_code;

  // Dummy data for sensor_data
  uint8_t sensor_data[4] = {0x01, 0x02, 0x03, 0x04};

  // Prepare manufacturer specific data with encrypted data
  ble_advdata_manuf_data_t manuf_specific_data;
  manuf_specific_data.data.p_data = sensor_data;
  manuf_specific_data.data.size = sizeof(sensor_data);

  // Prepare advertising data structure
  ble_advdata_t advdata;
  memset(&advdata, 0, sizeof(advdata));
  advdata.flags = BLE_GAP_ADV_FLAG_BR_EDR_NOT_SUPPORTED;
  advdata.p_manuf_specific_data = &manuf_specific_data;

  // Update the advertising data
  err_code = ble_advertising_advdata_update(&m_advertising, &advdata, NULL);
  APP_ERROR_CHECK(err_code);
}